@@ -33,11 +33,14 @@
 #endif
 
 #include <string.h>
+#include <sys/resource.h>
 
 #include <glib.h>
 #include <glib-object.h>
 #include <glib/gi18n-lib.h>
 
+#include "brasero-volume.h"
+
 #include "burn-basics.h"
 #include "burn-debug.h"
 #include "burn-plugin-manager.h"
@@ -70,13 +73,22 @@ struct _BraseroSessionCfgPrivate
 	BraseroBurnFlag supported;
 	BraseroBurnFlag compulsory;
 
+	/* Do some caching to improve performances */
+	BraseroImageFormat output_format;
 	gchar *output;
 
+	BraseroTrackType *source;
+	goffset disc_size;
+	goffset session_blocks;
+	goffset session_size;
+
 	BraseroSessionError is_valid;
 
 	guint CD_TEXT_modified:1;
 	guint configuring:1;
 	guint disabled:1;
+
+	guint output_msdos:1;
 };
 
 #define BRASERO_SESSION_CFG_PRIVATE(o)  (G_TYPE_INSTANCE_GET_PRIVATE ((o), BRASERO_TYPE_SESSION_CFG, BraseroSessionCfgPrivate))
@@ -114,7 +126,7 @@ brasero_session_cfg_tag_changed (BraseroBurnSession *session,
 
 /**
  * brasero_session_cfg_has_default_output_path:
- * @cfg: a #BraseroSessionCfg
+ * @session: a #BraseroSessionCfg
  *
  * This function returns whether the path returned
  * by brasero_burn_session_get_output () is an 
@@ -310,8 +322,10 @@ brasero_session_cfg_get_output_path (BraseroBurnSession *session,
 	BraseroBurnResult result;
 	BraseroImageFormat format;
 	BraseroBurnSessionClass *klass;
+	BraseroSessionCfgPrivate *priv;
 
 	klass = BRASERO_BURN_SESSION_CLASS (brasero_session_cfg_parent_class);
+	priv = BRASERO_SESSION_CFG_PRIVATE (session);
 
 	result = klass->get_output_path (session,
 					 image,
@@ -319,8 +333,13 @@ brasero_session_cfg_get_output_path (BraseroBurnSession *session,
 	if (result == BRASERO_BURN_OK)
 		return result;
 
-	format = brasero_burn_session_get_output_format (session);
-	path = brasero_image_format_get_default_path (format);
+	if (priv->output_format == BRASERO_IMAGE_FORMAT_NONE)
+		return BRASERO_BURN_ERR;
+
+	/* Note: path and format are determined earlier in fact, in the function
+	 * that check the free space on the hard drive. */
+	path = g_strdup (priv->output);
+	format = priv->output_format;
 
 	switch (format) {
 	case BRASERO_IMAGE_FORMAT_BIN:
@@ -340,6 +359,8 @@ brasero_session_cfg_get_output_path (BraseroBurnSession *session,
 
 	default:
 		g_free (path);
+		g_free (priv->output);
+		priv->output = NULL;
 		return BRASERO_BURN_ERR;
 	}
 
@@ -350,20 +371,28 @@ static BraseroImageFormat
 brasero_session_cfg_get_output_format (BraseroBurnSession *session)
 {
 	BraseroBurnSessionClass *klass;
+	BraseroSessionCfgPrivate *priv;
 	BraseroImageFormat format;
 
 	klass = BRASERO_BURN_SESSION_CLASS (brasero_session_cfg_parent_class);
 	format = klass->get_output_format (session);
 
-	if (format == BRASERO_IMAGE_FORMAT_NONE)
-		format = brasero_burn_session_get_default_output_format (session);
+	if (format != BRASERO_IMAGE_FORMAT_NONE)
+		return format;
 
-	return format;
+	priv = BRASERO_SESSION_CFG_PRIVATE (session);
+
+	if (priv->output_format)
+		return priv->output_format;
+
+	/* Cache the path for later use */
+	priv->output_format = brasero_burn_session_get_default_output_format (session);
+	return priv->output_format;
 }
 
 /**
  * brasero_session_cfg_get_error:
- * @cfg: a #BraseroSessionCfg
+ * @session: a #BraseroSessionCfg
  *
  * This function returns the current status and if
  * autoconfiguration is/was successful.
@@ -372,11 +401,11 @@ brasero_session_cfg_get_output_format (BraseroBurnSession *session)
  **/
 
 BraseroSessionError
-brasero_session_cfg_get_error (BraseroSessionCfg *self)
+brasero_session_cfg_get_error (BraseroSessionCfg *session)
 {
 	BraseroSessionCfgPrivate *priv;
 
-	priv = BRASERO_SESSION_CFG_PRIVATE (self);
+	priv = BRASERO_SESSION_CFG_PRIVATE (session);
 
 	if (priv->is_valid == BRASERO_SESSION_VALID
 	&&  priv->CD_TEXT_modified)
@@ -387,35 +416,35 @@ brasero_session_cfg_get_error (BraseroSessionCfg *self)
 
 /**
  * brasero_session_cfg_disable:
- * @cfg: a #BraseroSessionCfg
+ * @session: a #BraseroSessionCfg
  *
  * This function disables autoconfiguration
  *
  **/
 
 void
-brasero_session_cfg_disable (BraseroSessionCfg *self)
+brasero_session_cfg_disable (BraseroSessionCfg *session)
 {
 	BraseroSessionCfgPrivate *priv;
 
-	priv = BRASERO_SESSION_CFG_PRIVATE (self);
+	priv = BRASERO_SESSION_CFG_PRIVATE (session);
 	priv->disabled = TRUE;
 }
 
 /**
  * brasero_session_cfg_enable:
- * @cfg: a #BraseroSessionCfg
+ * @session: a #BraseroSessionCfg
  *
  * This function (re)-enables autoconfiguration
  *
  **/
 
 void
-brasero_session_cfg_enable (BraseroSessionCfg *self)
+brasero_session_cfg_enable (BraseroSessionCfg *session)
 {
 	BraseroSessionCfgPrivate *priv;
 
-	priv = BRASERO_SESSION_CFG_PRIVATE (self);
+	priv = BRASERO_SESSION_CFG_PRIVATE (session);
 	priv->disabled = FALSE;
 }
 
@@ -531,6 +560,16 @@ brasero_session_cfg_set_drive_properties_flags (BraseroSessionCfg *self,
 	priv = BRASERO_SESSION_CFG_PRIVATE (self);
 
 	original_flags = brasero_burn_session_get_flags (BRASERO_BURN_SESSION (self));
+
+	/* If the session is invalid no need to check the flags: just add them.
+	 * The correct flags will be re-computed anyway when the session becomes
+	 * valid again. */
+	if (priv->is_valid != BRASERO_SESSION_VALID) {
+		BRASERO_BURN_LOG ("Session currently not ready for flag computation: adding flags (will update later)");
+		brasero_burn_session_set_flags (BRASERO_BURN_SESSION (self), flags);
+		return;
+	}
+
 	BRASERO_BURN_LOG ("Resetting all flags");
 	BRASERO_BURN_LOG_FLAGS (original_flags, "Current are");
 	BRASERO_BURN_LOG_FLAGS (flags, "New should be");
@@ -668,6 +707,115 @@ brasero_session_cfg_check_drive_settings (BraseroSessionCfg *self)
 	brasero_session_cfg_add_drive_properties_flags (self, flags);
 }
 
+static BraseroSessionError
+brasero_session_cfg_check_volume_size (BraseroSessionCfg *self)
+{
+	struct rlimit limit;
+	BraseroSessionCfgPrivate *priv;
+
+	priv = BRASERO_SESSION_CFG_PRIVATE (self);
+	if (!priv->disc_size) {
+		GFileInfo *info;
+		gchar *directory;
+		GFile *file = NULL;
+		const gchar *filesystem;
+
+		/* Cache the path for later use */
+		if (priv->output_format == BRASERO_IMAGE_FORMAT_NONE)
+			priv->output_format = brasero_burn_session_get_output_format (BRASERO_BURN_SESSION (self));
+
+		if (!priv->output) {
+			gchar *name = NULL;
+
+			/* If we try to copy a volume get (and use) its name */
+			if (brasero_track_type_get_has_medium (priv->source)) {
+				BraseroMedium *medium;
+
+				medium = brasero_burn_session_get_src_medium (BRASERO_BURN_SESSION (self));
+				if (medium)
+					name = brasero_volume_get_name (BRASERO_VOLUME (medium));
+			}
+
+			priv->output = brasero_image_format_get_default_path (priv->output_format, name);
+			g_free (name);
+		}
+
+		directory = g_path_get_dirname (priv->output);
+		file = g_file_new_for_path (directory);
+		g_free (directory);
+
+		if (file == NULL)
+			goto error;
+
+		/* Check permissions first */
+		info = g_file_query_info (file,
+					  G_FILE_ATTRIBUTE_ACCESS_CAN_WRITE,
+					  G_FILE_QUERY_INFO_NONE,
+					  NULL,
+					  NULL);
+		if (!info) {
+			g_object_unref (file);
+			goto error;
+		}
+
+		if (!g_file_info_get_attribute_boolean (info, G_FILE_ATTRIBUTE_ACCESS_CAN_WRITE)) {
+			g_object_unref (info);
+			g_object_unref (file);
+			goto error;
+		}
+		g_object_unref (info);
+
+		/* Now size left */
+		info = g_file_query_filesystem_info (file,
+						     G_FILE_ATTRIBUTE_FILESYSTEM_FREE ","
+						     G_FILE_ATTRIBUTE_FILESYSTEM_TYPE,
+						     NULL,
+						     NULL);
+		g_object_unref (file);
+
+		if (!info)
+			goto error;
+
+		/* Now check the filesystem type: the problem here is that some
+		 * filesystems have a maximum file size limit of 4 GiB and more than
+		 * often we need a temporary file size of 4 GiB or more. */
+		filesystem = g_file_info_get_attribute_string (info, G_FILE_ATTRIBUTE_FILESYSTEM_TYPE);
+		if (!g_strcmp0 (filesystem, "msdos"))
+			priv->output_msdos = TRUE;
+		else
+			priv->output_msdos = FALSE;
+
+		priv->disc_size = g_file_info_get_attribute_uint64 (info, G_FILE_ATTRIBUTE_FILESYSTEM_FREE);
+		g_object_unref (info);
+	}
+
+	BRASERO_BURN_LOG ("Session size %lli/Hard drive size %lli",
+			  priv->session_size,
+			  priv->disc_size);
+
+	if (priv->output_msdos && priv->session_size >= 2147483648ULL)
+		goto error;
+
+	if (priv->session_size > priv->disc_size)
+		goto error;
+
+	/* Last but not least, use getrlimit () to check that we are allowed to
+	 * write a file of such length and that quotas won't get in our way */
+	if (getrlimit (RLIMIT_FSIZE, &limit))
+		goto error;
+
+	if (limit.rlim_cur < priv->session_size)
+		goto error;
+
+	priv->is_valid = BRASERO_SESSION_VALID;
+	return BRASERO_SESSION_VALID;
+
+error:
+
+	priv->is_valid = BRASERO_SESSION_INSUFFICIENT_SPACE;
+	return BRASERO_SESSION_INSUFFICIENT_SPACE;
+}
+
 static BraseroSessionError
 brasero_session_cfg_check_size (BraseroSessionCfg *self)
 {
@@ -677,23 +825,39 @@ brasero_session_cfg_check_size (BraseroSessionCfg *self)
 	BraseroDrive *burner;
 	GValue *value = NULL;
 	/* in sectors */
-	goffset session_size;
 	goffset max_sectors;
-	goffset disc_size;
 
 	priv = BRASERO_SESSION_CFG_PRIVATE (self);
 
+	/* Get the session size if need be */
+	if (!priv->session_blocks) {
+		if (brasero_burn_session_tag_lookup (BRASERO_BURN_SESSION (self),
+						     BRASERO_DATA_TRACK_SIZE_TAG,
+						     &value) == BRASERO_BURN_OK) {
+			priv->session_blocks = g_value_get_int64 (value);
+			priv->session_size = priv->session_blocks * 2048;
+		}
+		else if (brasero_burn_session_tag_lookup (BRASERO_BURN_SESSION (self),
+							  BRASERO_STREAM_TRACK_SIZE_TAG,
+							  &value) == BRASERO_BURN_OK) {
+			priv->session_blocks = g_value_get_int64 (value);
+			priv->session_size = priv->session_blocks * 2352;
+		}
+		else
+			brasero_burn_session_get_size (BRASERO_BURN_SESSION (self),
+						       &priv->session_blocks,
+						       &priv->session_size);
+	}
+
+	/* Get the disc and its size if need be */
 	burner = brasero_burn_session_get_burner (BRASERO_BURN_SESSION (self));
 	if (!burner) {
 		priv->is_valid = BRASERO_SESSION_NO_OUTPUT;
 		return BRASERO_SESSION_NO_OUTPUT;
 	}
 
-	/* FIXME: here we could check the hard drive space */
-	if (brasero_drive_is_fake (burner)) {
-		priv->is_valid = BRASERO_SESSION_VALID;
-		return BRASERO_SESSION_VALID;
-	}
+	if (brasero_drive_is_fake (burner))
+		return brasero_session_cfg_check_volume_size (self);
 
 	medium = brasero_drive_get_medium (burner);
 	if (!medium) {
@@ -701,33 +865,18 @@ brasero_session_cfg_check_size (BraseroSessionCfg *self)
 		return BRASERO_SESSION_NO_OUTPUT;
 	}
 
-	disc_size = brasero_burn_session_get_available_medium_space (BRASERO_BURN_SESSION (self));
-	if (disc_size < 0)
-		disc_size = 0;
-
-	/* get input track size */
-	session_size = 0;
-
-	if (brasero_burn_session_tag_lookup (BRASERO_BURN_SESSION (self),
-					     BRASERO_DATA_TRACK_SIZE_TAG,
-					     &value) == BRASERO_BURN_OK) {
-		session_size = g_value_get_int64 (value);
+	/* Get both sizes if need be */
+	if (!priv->disc_size) {
+		priv->disc_size = brasero_burn_session_get_available_medium_space (BRASERO_BURN_SESSION (self));
+		if (priv->disc_size < 0)
+			priv->disc_size = 0;
 	}
-	else if (brasero_burn_session_tag_lookup (BRASERO_BURN_SESSION (self),
-						  BRASERO_STREAM_TRACK_SIZE_TAG,
-						  &value) == BRASERO_BURN_OK) {
-		session_size = g_value_get_int64 (value);
-	}
-	else
-		brasero_burn_session_get_size (BRASERO_BURN_SESSION (self),
-					       &session_size,
-					       NULL);
 
 	BRASERO_BURN_LOG ("Session size %lli/Disc size %lli",
-			  session_size,
-			  disc_size);
+			  priv->session_blocks,
+			  priv->disc_size);
 
-	if (session_size < disc_size) {
+	if (priv->session_blocks < priv->disc_size) {
 		priv->is_valid = BRASERO_SESSION_VALID;
 		return BRASERO_SESSION_VALID;
 	}
@@ -743,8 +892,8 @@ brasero_session_cfg_check_size (BraseroSessionCfg *self)
 	 * when we propose overburning to the user, we could ask if he wants
 	 * us to determine how much data can be written to a particular disc
 	 * provided he has chosen a real disc. */
-	max_sectors = disc_size * 103 / 100;
-	if (max_sectors < session_size) {
+	max_sectors = priv->disc_size * 103 / 100;
+	if (max_sectors < priv->session_blocks) {
 		priv->is_valid = BRASERO_SESSION_INSUFFICIENT_SPACE;
 		return BRASERO_SESSION_INSUFFICIENT_SPACE;
 	}
@@ -787,19 +936,20 @@ brasero_session_cfg_set_tracks_audio_format (BraseroBurnSession *session,
 	}
 }
 
-static void
-brasero_session_cfg_update (BraseroSessionCfg *self)
+static gboolean
+brasero_session_cfg_can_update (BraseroSessionCfg *self)
 {
-	BraseroTrackType *source = NULL;
 	BraseroSessionCfgPrivate *priv;
 	BraseroBurnResult result;
 	BraseroStatus *status;
-	BraseroDrive *burner;
 
 	priv = BRASERO_SESSION_CFG_PRIVATE (self);
 
+	if (priv->disabled)
+		return FALSE;
+
 	if (priv->configuring)
-		return;
+		return FALSE;
 
 	/* Make sure the session is ready */
 	status = brasero_status_new ();
@@ -811,7 +961,7 @@ brasero_session_cfg_update (BraseroSessionCfg *self)
 		g_signal_emit (self,
 			       session_cfg_signals [IS_VALID_SIGNAL],
 			       0);
-		return;
+		return FALSE;
 	}
 
 	if (result == BRASERO_BURN_ERR) {
@@ -827,21 +977,35 @@ brasero_session_cfg_update (BraseroSessionCfg *self)
 				g_signal_emit (self,
 					       session_cfg_signals [IS_VALID_SIGNAL],
 					       0);
-				return;
+				return FALSE;
 			}
 
 			g_error_free (error);
 		}
 	}
 	g_object_unref (status);
+	return TRUE;
+}
+
+static void
+brasero_session_cfg_update (BraseroSessionCfg *self)
+{
+	BraseroSessionCfgPrivate *priv;
+	BraseroBurnResult result;
+	BraseroDrive *burner;
+
+	priv = BRASERO_SESSION_CFG_PRIVATE (self);
 
 	/* Make sure there is a source */
-	source = brasero_track_type_new ();
-	brasero_burn_session_get_input_type (BRASERO_BURN_SESSION (self), source);
+	if (priv->source) {
+		brasero_track_type_free (priv->source);
+		priv->source = NULL;
+	}
 
-	if (brasero_track_type_is_empty (source)) {
-		brasero_track_type_free (source);
+	priv->source = brasero_track_type_new ();
+	brasero_burn_session_get_input_type (BRASERO_BURN_SESSION (self), priv->source);
 
+	if (brasero_track_type_is_empty (priv->source)) {
 		priv->is_valid = BRASERO_SESSION_EMPTY;
 		g_signal_emit (self,
 			       session_cfg_signals [IS_VALID_SIGNAL],
@@ -850,10 +1014,8 @@ brasero_session_cfg_update (BraseroSessionCfg *self)
 	}
 
 	/* it can be empty with just an empty track */
-	if (brasero_track_type_get_has_medium (source)
-	&&  brasero_track_type_get_medium_type (source) == BRASERO_MEDIUM_NONE) {
-		brasero_track_type_free (source);
-
+	if (brasero_track_type_get_has_medium (priv->source)
+	&&  brasero_track_type_get_medium_type (priv->source) == BRASERO_MEDIUM_NONE) {
 		priv->is_valid = BRASERO_SESSION_NO_INPUT_MEDIUM;
 		g_signal_emit (self,
 			       session_cfg_signals [IS_VALID_SIGNAL],
@@ -861,13 +1023,11 @@ brasero_session_cfg_update (BraseroSessionCfg *self)
 		return;
 	}
 
-	if (brasero_track_type_get_has_image (source)
-	&&  brasero_track_type_get_image_format (source) == BRASERO_IMAGE_FORMAT_NONE) {
+	if (brasero_track_type_get_has_image (priv->source)
+	&&  brasero_track_type_get_image_format (priv->source) == BRASERO_IMAGE_FORMAT_NONE) {
 		gchar *uri;
 		GSList *tracks;
 
-		brasero_track_type_free (source);
-
 		tracks = brasero_burn_session_get_tracks (BRASERO_BURN_SESSION (self));
 
 		/* It can be two cases:
@@ -897,8 +1057,6 @@ brasero_session_cfg_update (BraseroSessionCfg *self)
 	/* make sure there is an output set */
 	burner = brasero_burn_session_get_burner (BRASERO_BURN_SESSION (self));
 	if (!burner) {
-		brasero_track_type_free (source);
-
 		priv->is_valid = BRASERO_SESSION_NO_OUTPUT;
 		g_signal_emit (self,
 			       session_cfg_signals [IS_VALID_SIGNAL],
@@ -906,46 +1064,56 @@ brasero_session_cfg_update (BraseroSessionCfg *self)
 		return;
 	}
 
+	/* In case the output was an image remove the path cache. It will be
+	 * re-computed on demand. */
+	if (priv->output) {
+		g_free (priv->output);
+		priv->output = NULL;
+	}
+
+	if (priv->output_format)
+		priv->output_format = BRASERO_IMAGE_FORMAT_NONE;
+
 	/* Check that current input and output work */
-	if (brasero_track_type_get_has_stream (source)) {
+	if (brasero_track_type_get_has_stream (priv->source)) {
 		if (priv->CD_TEXT_modified) {
 			/* Try to redo what we undid (after all a new plugin
 			 * could have been activated in the mean time ...) and
 			 * see what happens */
-			brasero_track_type_set_stream_format (source,
+			brasero_track_type_set_stream_format (priv->source,
 							      BRASERO_METADATA_INFO|
-							      brasero_track_type_get_stream_format (source));
-			result = brasero_burn_session_input_supported (BRASERO_BURN_SESSION (self), source, FALSE);
+							      brasero_track_type_get_stream_format (priv->source));
+			result = brasero_burn_session_input_supported (BRASERO_BURN_SESSION (self), priv->source, FALSE);
 			if (result == BRASERO_BURN_OK) {
 				priv->CD_TEXT_modified = FALSE;
 
 				priv->configuring = TRUE;
 				brasero_session_cfg_set_tracks_audio_format (BRASERO_BURN_SESSION (self),
-									     brasero_track_type_get_stream_format (source));
+									     brasero_track_type_get_stream_format (priv->source));
 				priv->configuring = FALSE;
 			}
 			else {
 				/* No, nothing's changed */
-				brasero_track_type_set_stream_format (source,
+				brasero_track_type_set_stream_format (priv->source,
 								      (~BRASERO_METADATA_INFO) &
-								      brasero_track_type_get_stream_format (source));
-				result = brasero_burn_session_input_supported (BRASERO_BURN_SESSION (self), source, FALSE);
+								      brasero_track_type_get_stream_format (priv->source));
+				result = brasero_burn_session_input_supported (BRASERO_BURN_SESSION (self), priv->source, FALSE);
 			}
 		}
 		else {
 			result = brasero_burn_session_can_burn (BRASERO_BURN_SESSION (self), FALSE);
 
 			if (result != BRASERO_BURN_OK
-			&& (brasero_track_type_get_stream_format (source) & BRASERO_METADATA_INFO)) {
+			&& (brasero_track_type_get_stream_format (priv->source) & BRASERO_METADATA_INFO)) {
 				/* Another special case in case some burning backends 
 				 * don't support CD-TEXT for audio (libburn). If no
 				 * other backend is available remove CD-TEXT option but
 				 * tell user... */
-				brasero_track_type_set_stream_format (source,
+				brasero_track_type_set_stream_format (priv->source,
 								      (~BRASERO_METADATA_INFO) &
-								      brasero_track_type_get_stream_format (source));
+								      brasero_track_type_get_stream_format (priv->source));
 
-				result = brasero_burn_session_input_supported (BRASERO_BURN_SESSION (self), source, FALSE);
+				result = brasero_burn_session_input_supported (BRASERO_BURN_SESSION (self), priv->source, FALSE);
 
 				BRASERO_BURN_LOG ("Tested support without Metadata information (result %d)", result);
 				if (result == BRASERO_BURN_OK) {
@@ -953,14 +1121,14 @@ brasero_session_cfg_update (BraseroSessionCfg *self)
 
 					priv->configuring = TRUE;
 					brasero_session_cfg_set_tracks_audio_format (BRASERO_BURN_SESSION (self),
-										     brasero_track_type_get_has_stream (source));
+										     brasero_track_type_get_has_stream (priv->source));
 					priv->configuring = FALSE;
 				}
 			}
 		}
 	}
-	else if (brasero_track_type_get_has_medium (source)
-	&&  (brasero_track_type_get_medium_type (source) & BRASERO_MEDIUM_HAS_AUDIO)) {
+	else if (brasero_track_type_get_has_medium (priv->source)
+	&&  (brasero_track_type_get_medium_type (priv->source) & BRASERO_MEDIUM_HAS_AUDIO)) {
 		BraseroImageFormat format = BRASERO_IMAGE_FORMAT_NONE;
 
 		/* If we copy an audio disc check the image
@@ -1013,9 +1181,9 @@ brasero_session_cfg_update (BraseroSessionCfg *self)
 	}
 
 	if (result != BRASERO_BURN_OK) {
-		if (brasero_track_type_get_has_medium (source)
-		&& (brasero_track_type_get_medium_type (source) & BRASERO_MEDIUM_PROTECTED)
-		&&  brasero_burn_library_input_supported (source) != BRASERO_BURN_OK) {
+		if (brasero_track_type_get_has_medium (priv->source)
+		&& (brasero_track_type_get_medium_type (priv->source) & BRASERO_MEDIUM_PROTECTED)
+		&&  brasero_burn_library_input_supported (priv->source) != BRASERO_BURN_OK) {
 			/* This is a special case to display a helpful message */
 			priv->is_valid = BRASERO_SESSION_DISC_PROTECTED;
 			g_signal_emit (self,
@@ -1029,13 +1197,12 @@ brasero_session_cfg_update (BraseroSessionCfg *self)
 				       0);
 		}
 
-		brasero_track_type_free (source);
 		return;
 	}
 
 	/* Special case for video projects */
-	if (brasero_track_type_get_has_stream (source)
-	&&  BRASERO_STREAM_FORMAT_HAS_VIDEO (brasero_track_type_get_stream_format (source))) {
+	if (brasero_track_type_get_has_stream (priv->source)
+	&&  BRASERO_STREAM_FORMAT_HAS_VIDEO (brasero_track_type_get_stream_format (priv->source))) {
 		/* Only set if it was not already set */
 		if (brasero_burn_session_tag_lookup (BRASERO_BURN_SESSION (self), BRASERO_VCD_TYPE, NULL) != BRASERO_BURN_OK)
 			brasero_burn_session_tag_add_int (BRASERO_BURN_SESSION (self),
@@ -1043,8 +1210,6 @@ brasero_session_cfg_update (BraseroSessionCfg *self)
 							  BRASERO_SVCD);
 	}
 
-	brasero_track_type_free (source);
-
 	/* Configure flags */
 	priv->configuring = TRUE;
 
@@ -1083,6 +1248,9 @@ brasero_session_cfg_session_loaded (BraseroTrackDataCfg *track,
 	priv = BRASERO_SESSION_CFG_PRIVATE (session);
 	if (priv->disabled)
 		return;
+	
+	priv->session_blocks = 0;
+	priv->session_size = 0;
 
 	session_flags = brasero_burn_session_get_flags (BRASERO_BURN_SESSION (session));
 	if (is_loaded) {
@@ -1103,10 +1271,13 @@ brasero_session_cfg_track_added (BraseroBurnSession *session,
 {
 	BraseroSessionCfgPrivate *priv;
 
-	priv = BRASERO_SESSION_CFG_PRIVATE (session);
-	if (priv->disabled)
+	if (!brasero_session_cfg_can_update (BRASERO_SESSION_CFG (session)))
 		return;
 
+	priv = BRASERO_SESSION_CFG_PRIVATE (session);
+	priv->session_blocks = 0;
+	priv->session_size = 0;
+
 	if (BRASERO_IS_TRACK_DATA_CFG (track))
 		g_signal_connect (track,
 				  "session-loaded",
@@ -1117,8 +1288,8 @@ brasero_session_cfg_track_added (BraseroBurnSession *session,
 	 * - check if all flags are supported
 	 * - check available formats for path
 	 * - set one path */
-	brasero_session_cfg_check_drive_settings (BRASERO_SESSION_CFG (session));
 	brasero_session_cfg_update (BRASERO_SESSION_CFG (session));
+	brasero_session_cfg_check_drive_settings (BRASERO_SESSION_CFG (session));
 }
 
 static void
@@ -1128,10 +1299,13 @@ brasero_session_cfg_track_removed (BraseroBurnSession *session,
 {
 	BraseroSessionCfgPrivate *priv;
 
-	priv = BRASERO_SESSION_CFG_PRIVATE (session);
-	if (priv->disabled)
+	if (!brasero_session_cfg_can_update (BRASERO_SESSION_CFG (session)))
 		return;
 
+	priv = BRASERO_SESSION_CFG_PRIVATE (session);
+	priv->session_blocks = 0;
+	priv->session_size = 0;
+
 	/* Just in case */
 	g_signal_handlers_disconnect_by_func (track,
 					      brasero_session_cfg_session_loaded,
@@ -1148,18 +1322,35 @@ brasero_session_cfg_track_changed (BraseroBurnSession *session,
 				   BraseroTrack *track)
 {
 	BraseroSessionCfgPrivate *priv;
+	BraseroTrackType *current;
+
+	if (!brasero_session_cfg_can_update (BRASERO_SESSION_CFG (session)))
+		return;
 
 	priv = BRASERO_SESSION_CFG_PRIVATE (session);
-	if (priv->disabled)
-		return;
+	priv->session_blocks = 0;
+	priv->session_size = 0;
+
+	current = brasero_track_type_new ();
+	brasero_burn_session_get_input_type (session, current);
+	if (brasero_track_type_equal (current, priv->source)) {
+		/* This is a shortcut if the source type has not changed */
+		brasero_track_type_free (current);
+		brasero_session_cfg_check_size (BRASERO_SESSION_CFG (session));
+		g_signal_emit (session,
+			       session_cfg_signals [IS_VALID_SIGNAL],
+			       0);
+ 		return;
+	}
+	brasero_track_type_free (current);
 
 	/* when that happens it's mostly because a medium source changed, or
 	 * a new image was set. 
 	 * - check if all flags are supported
 	 * - check available formats for path
 	 * - set one path if need be */
-	brasero_session_cfg_check_drive_settings (BRASERO_SESSION_CFG (session));
 	brasero_session_cfg_update (BRASERO_SESSION_CFG (session));
+	brasero_session_cfg_check_drive_settings (BRASERO_SESSION_CFG (session));
 }
 
 static void
@@ -1167,18 +1358,17 @@ brasero_session_cfg_output_changed (BraseroBurnSession *session,
 				    BraseroMedium *former)
 {
 	BraseroSessionCfgPrivate *priv;
-	BraseroTrackType *type;
 
-	priv = BRASERO_SESSION_CFG_PRIVATE (session);
-	if (priv->disabled)
+	if (!brasero_session_cfg_can_update (BRASERO_SESSION_CFG (session)))
 		return;
 
-	/* Case for video project */
-	type = brasero_track_type_new ();
-	brasero_burn_session_get_input_type (session, type);
+	priv = BRASERO_SESSION_CFG_PRIVATE (session);
+	priv->disc_size = 0;
 
-	if (brasero_track_type_get_has_stream (type)
-	&&  BRASERO_STREAM_FORMAT_HAS_VIDEO (brasero_track_type_get_stream_format (type))) {
+	/* Case for video project */
+	if (priv->source
+	&&  brasero_track_type_get_has_stream (priv->source)
+	&&  BRASERO_STREAM_FORMAT_HAS_VIDEO (brasero_track_type_get_stream_format (priv->source))) {
 		BraseroMedia media;
 
 		media = brasero_burn_session_get_dest_media (session);
@@ -1204,13 +1394,12 @@ brasero_session_cfg_output_changed (BraseroBurnSession *session,
 								  BRASERO_AUDIO_FORMAT_AC3);
 		}
 	}
-	brasero_track_type_free (type);
 
 	/* In this case need to :
 	 * - check if all flags are supported
 	 * - for images, set a path if it wasn't already set */
-	brasero_session_cfg_check_drive_settings (BRASERO_SESSION_CFG (session));
 	brasero_session_cfg_update (BRASERO_SESSION_CFG (session));
+	brasero_session_cfg_check_drive_settings (BRASERO_SESSION_CFG (session));
 }
 
 static void
@@ -1219,21 +1408,25 @@ brasero_session_cfg_caps_changed (BraseroPluginManager *manager,
 {
 	BraseroSessionCfgPrivate *priv;
 
-	priv = BRASERO_SESSION_CFG_PRIVATE (self);
-	if (priv->disabled)
+	if (!brasero_session_cfg_can_update (self))
 		return;
+ 
+	priv = BRASERO_SESSION_CFG_PRIVATE (self);
+	priv->disc_size = 0;
+	priv->session_blocks = 0;
+	priv->session_size = 0;
 
 	/* In this case we need to check if:
 	 * - flags are supported or not supported anymore
 	 * - image types as input/output are supported
 	 * - if the current set of input/output still works */
-	brasero_session_cfg_check_drive_settings (self);
 	brasero_session_cfg_update (self);
+	brasero_session_cfg_check_drive_settings (self);
 }
 
 /**
  * brasero_session_cfg_add_flags:
- * @cfg: a #BraseroSessionCfg
+ * @session: a #BraseroSessionCfg
  * @flags: a #BraseroBurnFlag
  *
  * Adds all flags from @flags that are supported.
@@ -1241,26 +1434,28 @@ brasero_session_cfg_caps_changed (BraseroPluginManager *manager,
  **/
 
 void
-brasero_session_cfg_add_flags (BraseroSessionCfg *self,
+brasero_session_cfg_add_flags (BraseroSessionCfg *session,
 			       BraseroBurnFlag flags)
 {
 	BraseroSessionCfgPrivate *priv;
 
-	priv = BRASERO_SESSION_CFG_PRIVATE (self);
+	priv = BRASERO_SESSION_CFG_PRIVATE (session);
 
 	if ((priv->supported & flags) != flags)
 		return;
 
-	if ((brasero_burn_session_get_flags (BRASERO_BURN_SESSION (self)) & flags) == flags)
+	if ((brasero_burn_session_get_flags (BRASERO_BURN_SESSION (session)) & flags) == flags)
 		return;
 
-	brasero_session_cfg_add_drive_properties_flags (self, flags);
-	brasero_session_cfg_update (self);
+	brasero_session_cfg_add_drive_properties_flags (session, flags);
+
+	if (brasero_session_cfg_can_update (session))
+		brasero_session_cfg_update (session);
 }
 
 /**
  * brasero_session_cfg_remove_flags:
- * @cfg: a #BraseroSessionCfg
+ * @session: a #BraseroSessionCfg
  * @flags: a #BraseroBurnFlag
  *
  * Removes all flags that are not compulsory.
@@ -1268,26 +1463,28 @@ brasero_session_cfg_add_flags (BraseroSessionCfg *self,
  **/
 
 void
-brasero_session_cfg_remove_flags (BraseroSessionCfg *self,
+brasero_session_cfg_remove_flags (BraseroSessionCfg *session,
 				  BraseroBurnFlag flags)
 {
 	BraseroSessionCfgPrivate *priv;
 
-	priv = BRASERO_SESSION_CFG_PRIVATE (self);
+	priv = BRASERO_SESSION_CFG_PRIVATE (session);
 
-	brasero_burn_session_remove_flag (BRASERO_BURN_SESSION (self), flags);
+	brasero_burn_session_remove_flag (BRASERO_BURN_SESSION (session), flags);
 
 	/* For this case reset all flags as some flags could
 	 * be made available after the removal of one flag
 	 * Example: After the removal of MULTI, FAST_BLANK
 	 * becomes available again for DVDRW sequential */
-	brasero_session_cfg_set_drive_properties_default_flags (self);
-	brasero_session_cfg_update (self);
+	brasero_session_cfg_set_drive_properties_default_flags (session);
+
+	if (brasero_session_cfg_can_update (session))
+		brasero_session_cfg_update (session);
 }
 
 /**
  * brasero_session_cfg_is_supported:
- * @cfg: a #BraseroSessionCfg
+ * @session: a #BraseroSessionCfg
  * @flag: a #BraseroBurnFlag
  *
  * Checks whether a particular flag is supported.
@@ -1297,18 +1494,18 @@ brasero_session_cfg_remove_flags (BraseroSessionCfg *self,
  **/
 
 gboolean
-brasero_session_cfg_is_supported (BraseroSessionCfg *self,
+brasero_session_cfg_is_supported (BraseroSessionCfg *session,
 				  BraseroBurnFlag flag)
 {
 	BraseroSessionCfgPrivate *priv;
 
-	priv = BRASERO_SESSION_CFG_PRIVATE (self);
+	priv = BRASERO_SESSION_CFG_PRIVATE (session);
 	return (priv->supported & flag) == flag;
 }
 
 /**
  * brasero_session_cfg_is_compulsory:
- * @cfg: a #BraseroSessionCfg
+ * @session: a #BraseroSessionCfg
  * @flag: a #BraseroBurnFlag
  *
  * Checks whether a particular flag is compulsory.
@@ -1318,12 +1515,12 @@ brasero_session_cfg_is_supported (BraseroSessionCfg *self,
  **/
 
 gboolean
-brasero_session_cfg_is_compulsory (BraseroSessionCfg *self,
+brasero_session_cfg_is_compulsory (BraseroSessionCfg *session,
 				   BraseroBurnFlag flag)
 {
 	BraseroSessionCfgPrivate *priv;
 
-	priv = BRASERO_SESSION_CFG_PRIVATE (self);
+	priv = BRASERO_SESSION_CFG_PRIVATE (session);
 	return (priv->compulsory & flag) == flag;
 }
 
@@ -1335,6 +1532,7 @@ brasero_session_cfg_init (BraseroSessionCfg *object)
 
 	priv = BRASERO_SESSION_CFG_PRIVATE (object);
 
+	priv->is_valid = BRASERO_SESSION_EMPTY;
 	manager = brasero_plugin_manager_get_default ();
 	g_signal_connect (manager,
 	                  "caps-changed",
@@ -1366,6 +1564,16 @@ brasero_session_cfg_finalize (GObject *object)
 	                                      brasero_session_cfg_caps_changed,
 	                                      object);
 
+	if (priv->source) {
+		brasero_track_type_free (priv->source);
+		priv->source = NULL;
+	}
+
+	if (priv->output) {
+		g_free (priv->output);
+		priv->output = NULL;
+	}
+
 	G_OBJECT_CLASS (brasero_session_cfg_parent_class)->finalize (object);
 }
 
