@@ -598,7 +598,7 @@ brasero_jacket_edit_init (BraseroJacketEdit *object)
 	gtk_toolbar_insert (GTK_TOOLBAR (toolbar), GTK_TOOL_ITEM (item), 0);
 
 	item = GTK_WIDGET (gtk_tool_button_new (NULL, _("Bac_kground Properties")));
-	gtk_tool_button_set_icon_name (GTK_TOOL_BUTTON (item), "background");
+	gtk_tool_button_set_icon_name (GTK_TOOL_BUTTON (item), "preferences-desktop-wallpaper");
 	gtk_tool_button_set_use_underline (GTK_TOOL_BUTTON (item), TRUE);
 	gtk_widget_set_tooltip_text (item, _("Background properties"));
 	gtk_widget_show (item);
@@ -837,8 +837,7 @@ brasero_jacket_edit_dialog_new (GtkWidget *toplevel,
 	window = gtk_dialog_new_with_buttons (_("Cover Editor"),
 					      GTK_WINDOW (toplevel),
 					      GTK_DIALOG_MODAL|
-					      GTK_DIALOG_DESTROY_WITH_PARENT|
-					      GTK_DIALOG_NO_SEPARATOR,
+					      GTK_DIALOG_DESTROY_WITH_PARENT,
 					      GTK_STOCK_CLOSE, GTK_RESPONSE_CLOSE,
 					      NULL);
 
