@@ -89,6 +89,7 @@
 #include "brasero-notify.h"
 #include "brasero-project-parse.h"
 #include "brasero-project-name.h"
+#include "brasero-drive-settings.h"
 
 static void brasero_project_class_init (BraseroProjectClass *klass);
 static void brasero_project_init (BraseroProject *sp);
@@ -1072,7 +1073,6 @@ brasero_project_init (BraseroProject *obj)
 
 	size_group = gtk_size_group_new (GTK_SIZE_GROUP_BOTH);
 
-	/* burn button set insensitive since there are no files in the selection */
 	obj->priv->burn = brasero_utils_make_button (_("_Burn…"),
 						     NULL,
 						     "media-optical-burn",
@@ -1163,7 +1163,6 @@ brasero_project_init (BraseroProject *obj)
 			    0);
 
 	g_object_unref (size_group);
-	
 }
 
 static void
@@ -1463,6 +1462,14 @@ brasero_project_setup_session (BraseroProject *project,
 	}
 }
 
+static void
+brasero_project_output_changed (BraseroBurnSession *session,
+                                BraseroMedium *former_medium,
+                                GtkDialog *dialog)
+{
+	gtk_dialog_response (dialog, GTK_RESPONSE_CANCEL);
+}
+
 static BraseroBurnResult
 brasero_project_drive_properties (BraseroProject *project)
 {
@@ -1474,6 +1481,7 @@ brasero_project_drive_properties (BraseroProject *project)
 	GtkWidget *options;
 	GtkWidget *button;
 	GtkWidget *dialog;
+	glong cancel_sig;
 	GtkWidget *box;
 	gchar *header;
 	gchar *string;
@@ -1488,12 +1496,17 @@ brasero_project_drive_properties (BraseroProject *project)
 	dialog = gtk_dialog_new_with_buttons (header,
 					      NULL,
 					      GTK_DIALOG_MODAL|
-					      GTK_DIALOG_NO_SEPARATOR|
 					      GTK_DIALOG_DESTROY_WITH_PARENT,
 					      GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
 					      NULL);
 	g_free (header);
 
+	/* This is in case the medium gets ejected instead of our locking it */
+	cancel_sig = g_signal_connect (project->priv->session,
+	                               "output-changed",
+	                               G_CALLBACK (brasero_project_output_changed),
+	                               dialog);
+
 	gtk_dialog_add_button (GTK_DIALOG (dialog),
 			       _("Burn _Several Copies"),
 			       GTK_RESPONSE_ACCEPT);
@@ -1538,6 +1551,8 @@ brasero_project_drive_properties (BraseroProject *project)
 	answer = gtk_dialog_run (GTK_DIALOG (dialog));
 	gtk_widget_destroy (dialog);
 
+	g_signal_handler_disconnect (project->priv->session, cancel_sig);
+
 	if (answer == GTK_RESPONSE_OK)
 		return BRASERO_BURN_OK;
 
@@ -1600,11 +1615,12 @@ brasero_project_image_properties (BraseroProject *project)
 	return (answer == GTK_RESPONSE_OK) ? BRASERO_BURN_OK:BRASERO_BURN_ERR;
 }
 
-void
+static void
 brasero_project_burn (BraseroProject *project)
 {
 	BraseroBurnResult res;
 	BraseroDisc *current_disc;
+	BraseroDriveSettings *settings;
 
 	/* Check that we are ready */
 	if (brasero_project_check_status (project) != BRASERO_BURN_OK)
@@ -1614,13 +1630,20 @@ brasero_project_burn (BraseroProject *project)
 	if (brasero_project_check_plugins_not_ready (project, BRASERO_BURN_SESSION (project->priv->session)) != BRASERO_BURN_OK)
 		return;
 
+	/* Set saved parameters for the session */
+	settings = brasero_drive_settings_new ();
+	brasero_drive_settings_set_session (settings, BRASERO_BURN_SESSION (project->priv->session));
+
 	if (!brasero_burn_session_is_dest_file (BRASERO_BURN_SESSION (project->priv->session)))
 		res = brasero_project_drive_properties (project);
 	else
 		res = brasero_project_image_properties (project);
 
-	if (res != BRASERO_BURN_OK && res != BRASERO_BURN_RETRY)
+	if (res != BRASERO_BURN_OK
+	&&  res != BRASERO_BURN_RETRY) {
+		g_object_unref (settings);
 		return;
+	}
 
 	project->priv->is_burning = 1;
 
@@ -1640,6 +1663,8 @@ brasero_project_burn (BraseroProject *project)
 						 BRASERO_BURN_SESSION (project->priv->session),
 						 res == BRASERO_BURN_RETRY);
 
+	g_object_unref (settings);
+
 	/* empty the stack of temporary tracks */
 	while (brasero_burn_session_pop_tracks (BRASERO_BURN_SESSION (project->priv->session)) == BRASERO_BURN_RETRY);
 
@@ -1866,7 +1891,7 @@ brasero_project_confirm_switch (BraseroProject *project,
 			return TRUE;
 
 		dialog = brasero_app_dialog (brasero_app_get_default (),
-					     _("Do you really want to create a new project and discard the changes to current one?"),
+					     _("Do you really want to create a new project and discard the current one?"),
 					     GTK_BUTTONS_CANCEL,
 					     GTK_MESSAGE_WARNING);
 
@@ -1995,6 +2020,9 @@ void
 brasero_project_set_source (BraseroProject *project,
 			    BraseroURIContainer *source)
 {
+	if (project->priv->chooser)
+		gtk_dialog_response (GTK_DIALOG (project->priv->chooser), GTK_RESPONSE_CANCEL);
+
 	if (project->priv->activated_id) {
 		g_signal_handler_disconnect (project->priv->current_source,
 					     project->priv->activated_id);
@@ -2120,7 +2148,7 @@ brasero_project_update_preview (GtkFileChooser *chooser,
 {
 	gchar *uri;
 
-	gtk_file_chooser_set_preview_widget_active (chooser, FALSE);
+	gtk_file_chooser_set_preview_widget_active (chooser, TRUE);
 
 	uri = gtk_file_chooser_get_preview_uri (chooser);
 	brasero_player_set_uri (player, uri);
@@ -2227,7 +2255,7 @@ brasero_project_add_uris_cb (GtkAction *action,
 	player = brasero_player_new ();
 
 	gtk_widget_show (player);
-	gtk_file_chooser_set_preview_widget_active (GTK_FILE_CHOOSER (project->priv->chooser), FALSE);
+	gtk_file_chooser_set_preview_widget_active (GTK_FILE_CHOOSER (project->priv->chooser), TRUE);
 	gtk_file_chooser_set_use_preview_label (GTK_FILE_CHOOSER (project->priv->chooser), FALSE);
 	gtk_file_chooser_set_preview_widget (GTK_FILE_CHOOSER (project->priv->chooser), player);
 
@@ -2629,7 +2657,7 @@ brasero_project_save_project_real (BraseroProject *project,
 				   const gchar *uri,
 				   BraseroProjectSave save_type)
 {
-	BraseroDiscResult result;
+	BraseroBurnResult result;
 	BraseroProjectType type;
 
 	g_return_val_if_fail (uri != NULL || project->priv->project != NULL, FALSE);
@@ -2638,19 +2666,6 @@ brasero_project_save_project_real (BraseroProject *project,
 	if (result != BRASERO_BURN_OK)
 		return FALSE;
 
-	if (result == BRASERO_DISC_ERROR_EMPTY_SELECTION) {
-		if (BRASERO_IS_AUDIO_DISC (project->priv->current))
-			brasero_project_no_song_dialog (project);
-		else if (BRASERO_IS_DATA_DISC (project->priv->current))
-			brasero_project_no_file_dialog (project);
-
-		return FALSE;
-	}
-	else if (result != BRASERO_DISC_OK) {
-		brasero_project_not_saved_dialog (project);
-		return FALSE;
-	}
-
 	brasero_project_setup_session (project, BRASERO_BURN_SESSION (project->priv->session));
         type = brasero_project_get_session_type (project);
 
@@ -2715,18 +2730,18 @@ brasero_project_save_project_ask_for_path (BraseroProject *project,
 	 * in plain text a list of the current displayed songs (only in save as
 	 * mode) */
 	if (type && BRASERO_IS_AUDIO_DISC (project->priv->current)) {
-		combo = gtk_combo_box_new_text ();
+		combo = gtk_combo_box_text_new ();
 		gtk_widget_show (combo);
 
-		gtk_combo_box_append_text (GTK_COMBO_BOX (combo), _("Save project as a Brasero audio project"));
-		gtk_combo_box_append_text (GTK_COMBO_BOX (combo), _("Save project as a plain text list"));
+		gtk_combo_box_text_append_text (GTK_COMBO_BOX_TEXT (combo), _("Save project as a Brasero audio project"));
+		gtk_combo_box_text_append_text (GTK_COMBO_BOX_TEXT (combo), _("Save project as a plain text list"));
 
 #ifdef BUILD_PLAYLIST
 
-		gtk_combo_box_append_text (GTK_COMBO_BOX (combo), _("Save project as a PLS playlist"));
-		gtk_combo_box_append_text (GTK_COMBO_BOX (combo), _("Save project as an M3U playlist"));
-		gtk_combo_box_append_text (GTK_COMBO_BOX (combo), _("Save project as an XSPF playlist"));
-		gtk_combo_box_append_text (GTK_COMBO_BOX (combo), _("Save project as an iriver playlist"));
+		gtk_combo_box_text_append_text (GTK_COMBO_BOX_TEXT (combo), _("Save project as a PLS playlist"));
+		gtk_combo_box_text_append_text (GTK_COMBO_BOX_TEXT (combo), _("Save project as an M3U playlist"));
+		gtk_combo_box_text_append_text (GTK_COMBO_BOX_TEXT (combo), _("Save project as an XSPF playlist"));
+		gtk_combo_box_text_append_text (GTK_COMBO_BOX_TEXT (combo), _("Save project as an iriver playlist"));
 
 #endif
 
